package eu.etaxonomy.taxeditor.prototype2.view;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.Activator;

public class MoveDialogView extends Dialog {

	private Text searchText;
	private TaxonomicTreeViewer treeViewer;
	private Label selectedTaxonNameLabel;
	private Taxon taxon;
	
	private List<Button> buttons = new ArrayList<Button>();
	
	public int open(Taxon taxon) {
		this.taxon = taxon;
		return super.open();
	}
	/**
	 * Create the dialog
	 * @param parentShell
	 */
	public MoveDialogView(Shell parentShell) {
		super(parentShell);
	}

	/**
	 * Create contents of the dialog
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		parent.setRedraw(true);
		Composite container = (Composite) super.createDialogArea(parent);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		container.setLayout(gridLayout);

		final Label youHaveChosenLabel = new Label(container, SWT.WRAP);
		final GridData gd_youHaveChosenLabel = new GridData(SWT.LEFT, SWT.FILL, true, false, 2, 1);
//		gd_youHaveChosenLabel.heightHint = 30;
		youHaveChosenLabel.setLayoutData(gd_youHaveChosenLabel);
//		youHaveChosenLabel.setText("You have chosen to turn \"" + taxon.getName().getNameCache() +
//				"\" into a synonym. Please choose a new accepted name from one of the following options.");
		youHaveChosenLabel.setText("Choose a taxonomic parent for \"" + taxon.getName().getTitleCache() +
		"\".");		

		final Group group_1 = new Group(container, SWT.NONE);
		group_1.setLayout(new GridLayout());
		final GridData gd_group_1 = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd_group_1.heightHint = 83;
		group_1.setLayoutData(gd_group_1);

		final Button chooseAcceptedNameButton = new Button(group_1, SWT.RADIO);
		buttons.add(chooseAcceptedNameButton);
		chooseAcceptedNameButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		chooseAcceptedNameButton.setText("Choose from taxonomic tree");

		treeViewer = new TaxonomicTreeViewer(group_1);
		treeViewer.getTree().addFocusListener(new SelectButtonFocusListener(chooseAcceptedNameButton));
		treeViewer.addDoubleClickListener(new SelectTaxonDoubleClickListener()); 

		final Composite composite = new Composite(container, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		composite.setLayout(new FillLayout(SWT.VERTICAL | SWT.HORIZONTAL));
		
		final Group group = new Group(composite, SWT.NONE);
		group.setLayout(new GridLayout());

		final Button chooseAcceptedTaxonButton = new Button(group, SWT.RADIO);
		buttons.add(chooseAcceptedTaxonButton);
		chooseAcceptedTaxonButton.setText("Choose from recently viewed taxa");

		TableViewer recentNamesViewer = new RecentNamesTableViewer(group);
		recentNamesViewer.addDoubleClickListener(new SelectTaxonDoubleClickListener());
		recentNamesViewer.getTable().addFocusListener(new SelectButtonFocusListener(chooseAcceptedTaxonButton));
		
		final Group group_2 = new Group(composite, SWT.NONE);
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		group_2.setLayout(gridLayout_1);

		final Button searchForNameButton = new Button(group_2, SWT.RADIO);
		buttons.add(searchForNameButton);
		searchForNameButton.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false));
		searchForNameButton.setText("Search for name of accepted taxon");
		new Label(group_2, SWT.NONE);
		
		searchText = new Text(group_2, SWT.BORDER);
		searchText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		searchText.setForeground(SWTResourceManager.getColor(192, 192, 192));
		searchText.setText("Use \"*\" for wildcard searching");
		searchText.addFocusListener(new SelectButtonFocusListener(searchForNameButton) {

			public void focusGained(FocusEvent e) {
				selectButton(button);
				searchText.setForeground(SWTResourceManager.getColor(0,0,0));
				searchText.setText("");
			}

			public void focusLost(FocusEvent e) {
				if (searchText.getText() == "") {
					searchText.setForeground(SWTResourceManager.getColor(192, 192, 192));
					searchText.setText("Use \"*\" for wildcard searching");					
				}
			}
			
		});

		final Button searchButton = new Button(group_2, SWT.NONE);
		searchButton.setLayoutData(new GridData());
		searchButton.setText("Search");

		final Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 2, 1));
		composite_1.setLayout(new FillLayout());

		new Label(composite_1, SWT.NONE).setText("Selected taxon: ");

		selectedTaxonNameLabel = new Label(composite_1, SWT.RIGHT);
		selectedTaxonNameLabel.setFont(SWTResourceManager.getFont("", 9, SWT.BOLD));
		selectedTaxonNameLabel.setText("");
		new Label(container, SWT.NONE);
		
		
		//
		initButtons();
		
		return container;
	}

	/**
	 * Create contents of the button bar
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	private void initButtons() {
		for ( Button button : buttons) {
			button.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(final SelectionEvent e) {
					selectButton((Button) e.getSource());
				}
			});			
		}
	}
	
	/**
	 * Since buttons are not in the same layout group, the others are not automatically 
	 *  deselected when one is selected. This method takes care of that.
	 * @param button
	 */
	private void selectButton(Button selectedButton) {
		for ( Button button : buttons) {
			if (button == selectedButton)
				button.setSelection(true);
			else
				button.setSelection(false);
		}
	}
	
	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(700, 500);
	}
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/edit_16x16.ico"));
		newShell.setText("Choose parent taxon for \"" + taxon.getName().getTitleCache() +
				"\"");
	}

	class SelectButtonFocusListener implements FocusListener {

		Button button; // the button that should be selected
		
		SelectButtonFocusListener(Button button) {
			this.button = button;
		}
		
		public void focusGained(FocusEvent e) {
			selectButton(button);
		}

		public void focusLost(FocusEvent e) {			
		}
		
	}
	
	class SelectTaxonDoubleClickListener implements IDoubleClickListener {

		public void doubleClick(DoubleClickEvent event) {								
//			TaxonName taxonname = null;
			Taxon taxon = null;
			try {
				taxon = (Taxon) ((StructuredSelection)event.getSelection()).getFirstElement();
			}catch (Exception e){
				e.printStackTrace();
				taxon = null;
			}
			selectedTaxonNameLabel.setText(taxon.getName().getTitleCache());
		}
	}
}