package eu.etaxonomy.taxeditor.prototype2.view;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.observable.set.ISetChangeListener;
import org.eclipse.core.databinding.observable.set.SetChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.controller.TaxonTransfer;
import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.GroupComposite;
import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.NameComposite;
import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.TesterooWritableSet;

public class Testeroo {

	private static final Logger logger = Logger.getLogger(Testeroo.class);
	
	protected Shell shell;
	/**
	 * Launch the application
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			Testeroo window = new Testeroo();
			window.open();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Open the window
	 */
	public void open() {
		final Display display = Display.getDefault();
		createContents(display);
		shell.open();
		shell.layout();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}
	Taxon taxon;
	Composite composite;
	/**
	 * Create contents of the window
	 */
	protected void createContents(Display display) {
		if (logger.isDebugEnabled()){  logger.debug("Creating contents" );}
				
		shell = new Shell();
		shell.setLayout(new FillLayout());
		shell.setSize(500, 375);
		shell.setText("SWT Application");
		
		composite = new Composite(shell, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridLayout.verticalSpacing = 10;
		composite.setLayout(gridLayout);
		composite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		
		taxon = new Taxon();
		taxon.setName(BotanicalName.PARSED_NAME("Accepted name"));
				
		GroupComposite homoGroupComposite = new GroupComposite(composite);
		homoGroupComposite.setTaxon(taxon);
		
		new NameComposite(homoGroupComposite, taxon);
		
		Synonym homosyn = new Synonym();
		
		homosyn.setName(BotanicalName.PARSED_NAME("Homotypic synonym"));
		taxon.addSynonym(homosyn, SynonymRelationshipType.HOMOTYPIC_SYNONYM_OF());
		new NameComposite(homoGroupComposite, homosyn, taxon);
	
		homoGroupComposite.drawRect();
		
		createSynAndHetGroup("Heterotypic synonym 1");
		createSynAndHetGroup("Heterotypic synonym 2");
		createSynAndHetGroup("Heterotypic synonym 3");
		createSynAndHetGroup("Heterotypic synonym 4");

		TesterooWritableSet.getInstance().addSetChangeListener
		(new ISetChangeListener() {
			public void handleSetChange(SetChangeEvent event) {
				for ( Object addition: event.diff.getAdditions()) {
					if (addition instanceof Synonym)
						createHeterotypicalGroup((Synonym) addition);
				}
				composite.layout();
			}
		});
		
		// Drop functionality
		Transfer[] types = new Transfer[] {TaxonTransfer.getInstance()};
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT;
		DropTarget target = new DropTarget(composite, operations);
		target.setTransfer(types);
		target.addDropListener(dropTargetAdapter);
		
		composite.addListener(SWT.Modify, new Listener() {

			public void handleEvent(Event event) {
				System.out.println("!");
				
			}

			
		});
				
	}
	
	private void createSynAndHetGroup(String synonymNameCache) {
		Synonym heterosyn = new Synonym();
		heterosyn.setName(BotanicalName.PARSED_NAME(synonymNameCache));	
		taxon.addSynonym(heterosyn, SynonymRelationshipType.HETEROTYPIC_SYNONYM_OF());
				
		createHeterotypicalGroup(heterosyn);
	}
	
	private void createHeterotypicalGroup(Synonym heterosyn) {

		GroupComposite heteroGroupComposite = new GroupComposite(composite);
		heteroGroupComposite.setTaxon(taxon);
		
		new NameComposite(heteroGroupComposite, heterosyn, taxon).setFocus();

		heteroGroupComposite.layout();
		heteroGroupComposite.drawRect();
		heteroGroupComposite.layout();
	}
	
	DropTargetAdapter dropTargetAdapter = new DropTargetAdapter() {
		
		public void drop(DropTargetEvent event) {
			
			Synonym synonym = 
					(Synonym) event.data; 
			
			GroupComposite heteroGroupComposite = new GroupComposite(composite);
			heteroGroupComposite.setTaxon(taxon);
			
			new NameComposite(heteroGroupComposite, synonym, taxon).setFocus();
			
			Testeroo.this.composite.layout();
		}		
	};
}

