package eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport;

import java.util.Scanner;

import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionAddSynonymToTaxon;

/**
 * Checks text entered into a name viewer for line breaks, creates
 *  new synonyms accordingly
 * 
 * @author p.ciardelli
 */
public class NameViewerVerifyListener implements VerifyListener {
	
	private Taxon taxon;

	NameViewerVerifyListener(Taxon taxon) {
		this.taxon = taxon;
	}
	
	/* 
	 * Checks for 3 conditions of name viewer text:
	 *   1) no line breaks in text
	 *   2) line break at the end of the text
	 *   3) line break in the middle of the text
	 *   
	 * (non-Javadoc)
	 * @see org.eclipse.swt.events.VerifyListener#verifyText(org.eclipse.swt.events.VerifyEvent)
	 */
	public void verifyText(VerifyEvent e) {
				
		String verifyText = e.text;
		
		StyledText styledText = (StyledText)e.widget;
		String widgetText = styledText.getText(); 
		int cursorPosition = styledText.getCaretOffset();
		
		// Is there a line break in verifyText?
		if (!verifyText.contains(System.getProperty("line.separator")))
			// Parse text normally
			return;
		
		// Don't add verifyText without further processing
		e.doit = false;
		
		// If user has entered return at the end of the line,
		//  make a new, empty synonym
		if (widgetText.length() == cursorPosition &&
				verifyText.equals(System.getProperty("line.separator"))) {
			new ActionAddSynonymToTaxon(taxon, "").run();
			return;
		}
		
		// Concatenate old and new texts
		String textForSplitting = widgetText.substring(0, cursorPosition) 
				+ verifyText
				+ widgetText.substring(cursorPosition);
		
		// Split on line breaks
		Scanner scanner = new Scanner( textForSplitting );
		scanner.useDelimiter (System.getProperty("line.separator"));
		
		// Put first string into name viewer
		styledText.setText(scanner.next());	
		
		// Start new synonyms with the rest
		while (scanner.hasNext())
			new ActionAddSynonymToTaxon(taxon, scanner.next()).run();
	}
}
