/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.actions.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.operations.name.CreateConceptRelationOperation;
import eu.etaxonomy.taxeditor.operations.name.CreateMisappliedNameOperation;

/**
 * @author p.ciardelli
 * @created 21.01.2009
 * @version 1.0
 */
public class CreateConceptRelationAction extends Action {
	private static final Logger logger = Logger
			.getLogger(CreateConceptRelationAction.class);
	
	private static String text = "Create concept relation";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageRegistry().getDescriptor(ITaxEditorConstants.CONCEPT_ICON);

	private Taxon taxon;
	private TaxonRelationshipType taxonRelationshipType;

	private CreateConceptRelationAction() {
		super(text);
		setImageDescriptor(image);
	}

	public CreateConceptRelationAction(Taxon taxon, TaxonRelationshipType taxonRelationshipType) {
		this();
		
		// causes LL error
		setText(taxonRelationshipType.getLabel());
		
		this.taxon = taxon;
		this.taxonRelationshipType = taxonRelationshipType;
	}

	public void run() {
		IUndoableOperation operation = new CreateConceptRelationOperation(getText(), 
				EditorController.getUndoContext(taxon), taxon, taxonRelationshipType);
		
		GlobalController.executeOperation(operation);
	}
	
}
