/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.operations.ChangeTaxonToSynonymOperation;

/**
 * @author p.ciardelli
 * @created 26.05.2008
 * @version 1.0
 */
public class ChangeTaxonToSynonymAction extends Action {
	private static final Logger logger = Logger
			.getLogger(ChangeTaxonToSynonymAction.class);

	private static String text = "Change this taxon to a synonym";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageRegistry().getDescriptor(ITaxEditorConstants.TAXON_TO_SYNONYM_ICON);

	private Taxon taxon;

	private ChangeTaxonToSynonymAction() {
		super(text);
		setImageDescriptor(image);
	}

	public ChangeTaxonToSynonymAction(Taxon taxon) {
		this();
		this.taxon = taxon;
	}

	public void run() {
		IUndoableOperation operation = new ChangeTaxonToSynonymOperation(getText(), 
				GlobalController.getWorkbenchUndoContext(), taxon);
		
		GlobalController.executeOperation(operation);
	}
}