/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;

/**
 * @author p.ciardelli
 * @created 26.01.2009
 * @version 1.0
 */
public class ConceptComposite extends NameComposite {
	private static final Logger logger = Logger
			.getLogger(ConceptComposite.class);
	
	private Taxon relatedTaxon;
	private TaxonRelationshipType relationshipType;
	
	/**
	 * 
	 */
	private boolean acceptedTaxonIsFromTaxon;
		
	private ConceptComposite(Composite parent, IManagedForm form,
			Taxon taxon, Taxon relatedTaxon, TaxonRelationshipType relationshipType, 
			boolean acceptedTaxonIsFromTaxon) {
		super(parent, form, NameComposite.CONCEPTRELATION, relatedTaxon);
		
		this.taxon = taxon;
		this.relatedTaxon = relatedTaxon;
		this.relationshipType = relationshipType;
		this.acceptedTaxonIsFromTaxon = acceptedTaxonIsFromTaxon;
		
		setDraggable(true);
		setIcon(CONCEPT_ICON);
		setFont(getViewerFont());
		setIndent(CONCEPT_INDENT);

		createMenu();
		
		initNameViewer(relatedTaxon);
	}
	
	public static ConceptComposite getNewInstance(Composite parent, IManagedForm form,
			Taxon taxon, TaxonRelationship relationship){

		TaxonRelationshipType relationshipType = relationship.getType();
		
		if (relationship.getToTaxon().equals(taxon)) {
			return new ConceptComposite (parent, form, taxon, 
					relationship.getFromTaxon(), relationshipType, false);
		} else {
			return new ConceptComposite (parent, form, taxon, 
					relationship.getToTaxon(), relationshipType, true);
		} 
	}

	private void createMenu() {
		// TODO Auto-generated method stub
		
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#getViewerFont()
	 */
	@Override
	protected Font getViewerFont() {
		return CONCEPT_FONT;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.IHasPropertySource#getPropertySource()
	 */
	@Override
	public IPropertySource getPropertySource() {
		// TODO Auto-generated method stub
		return null;
	}

	public Taxon getRelatedTaxon() {
		return relatedTaxon;
	}
	
	protected TaxonNameBase getName() {
		return relatedTaxon.getName();
	}
	
}






