package eu.etaxonomy.taxeditor.navigation;

import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.actions.TaxonTransfer;
import eu.etaxonomy.taxeditor.actions.ui.OpenTaxonEditorAction;

public class FavoritesTableViewer extends TableViewer {

	public FavoritesTableViewer(Composite parent) {
		super(parent, SWT.H_SCROLL | SWT.V_SCROLL);
		this.getTable().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		// Add content provider
		ObservableListContentProvider favoritesContentProviderList = new ObservableListContentProvider();
		this.setContentProvider(favoritesContentProviderList);
		
		// Label provider that listens for changes to name cache
		IObservableMap[] rnViewerLabelProviderMaps = BeansObservables.observeMaps(favoritesContentProviderList.
//				getKnownElements(), IFavoritesDraggable.class, new String[]{"label"});
				getKnownElements(), Taxon.class, new String[]{"name"});
		this.setLabelProvider(new ObservableMapLabelProvider(rnViewerLabelProviderMaps) {
			
			/*
			 * JFace databinding syntax makes it to difficult to retrieve
			 * 	Taxon.getName().getNameCache, so override function that returns
			 *  label
			 * 
			 *  (non-Javadoc)
			 * @see org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider#getColumnText(java.lang.Object, int)
			 */
			public String getColumnText(Object element, int columnIndex) {
				return element.toString();
//				return ((Taxon) element).getName().getTitleCache();
			}			
		});
		
		// Listens for new taxa opened for editing
		this.setInput(FavoritesView.getObservableFavorites());
		
		// On double click, open name editor
		this.addDoubleClickListener(new IDoubleClickListener(){

			public void doubleClick(DoubleClickEvent event) {
				Object element = ((StructuredSelection)event.getSelection()).getFirstElement();
				if (element instanceof Taxon) {
					new OpenTaxonEditorAction((Taxon) element).run();
				}
			}
		});
		
		// Drop functionality
		Transfer[] types = new Transfer[] {TaxonTransfer.getInstance()};
//		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT;
		int operations = DND.DROP_COPY;
		DropTarget target = new DropTarget(this.getControl(), operations);
		target.setTransfer(types);
		target.addDropListener(new DropTargetAdapter() {
			
			public void dragEnter(DropTargetEvent event) {
			}
			
			public void drop(DropTargetEvent event) {
				
				// Add event to favorites list		
				FavoritesView.addFavorite(event.data);
			}
		});
	}
}